@@ -54,6 +54,8 @@ class AutoSaltSLS(object):
         self.source_settings = source_settings
         self.source_url_root = source_url_root
         self.full_filename = None
+        self.format = None
+        self.hidden = False
 
         # Build the full filename
         if self.filename:
@@ -223,18 +225,29 @@ class AutoSaltSLS(object):
             with open(self.full_filename) as sls_file:
                 entry = None
                 included = False
+                line_no = 0
 
                 # Read each line
                 for line in sls_file:
-                    # Skip lines starting with comment ignore prefix (e.g. '#!')
-                    if line.startswith(self.source_settings.comment_ignore_prefix):
-                        continue
+                    line_no += 1
 
                     # Remove the newline
                     line = line.strip("\n")
 
+                    # Grab the file format from the first line of the file
+                    if line.startswith("#!") and line_no == 1:
+                        self.format = line.replace("#!", "").strip()
+
+                    # Skip lines starting with comment ignore prefix (e.g. '#!')
+                    if self._check_line_startswith(
+                        line, self.source_settings.comment_ignore_prefix
+                    ):
+                        continue
+
                     # Start a block and create an AutoSaltSLSEntry object when we get the doc prefix (e.g. '###')
-                    if line.startswith(self.source_settings.doc_prefix):
+                    if self._check_line_startswith(
+                        line, self.source_settings.doc_prefix
+                    ):
                         # Finish any current entry and store it in case we have two concurrent blocks without
                         # any lines in between
                         if entry:
@@ -251,49 +264,97 @@ class AutoSaltSLS(object):
                             directives = [x.strip() for x in line.split(",")]
 
                             # Process directives
+                            ignore = False
                             for directive in directives:
-                                # 'topfile' is an sls directive
-                                if directive == "topfile":
+                                # 'hidden' marks the file as not to have documentation generated so we might as well
+                                # treat it as 'ignore' too
+                                if "hidden" in directives:
                                     logger.debug(
-                                        "[AutoSaltSLS] Marking sls {0} as top file".format(
+                                        "[AutoSaltSLS] Marking sls {0} as hidden due to directive".format(
+                                            self.basename
+                                        )
+                                    )
+                                    self.hidden = True
+                                    ignore = True
+                                # 'ignore' halts all processing for this file
+                                elif directive == "ignore":
+                                    logger.debug(
+                                        "[AutoSaltSLS] Aborting processing of sls {0} due to 'ignore' directive".format(
+                                            self.basename
+                                        )
+                                    )
+                                    ignore = True
+                                # 'topfile' is an sls directive
+                                elif directive == "topfile":
+                                    logger.debug(
+                                        "[AutoSaltSLS] Marking sls {0} as top file due to directive".format(
                                             self.basename
                                         )
                                     )
                                     self.topfile = True
-
                                 # Everything else is for an entry
                                 elif hasattr(entry, directive):
                                     setattr(entry, directive, True)
+
+                            # Check the ignore flag and stop processing
+                            if ignore:
+                                break
                         continue
 
                     # End the block
-                    if entry and not line.startswith(
-                        self.source_settings.comment_prefix
+                    if entry and not self._check_line_startswith(
+                        line, self.source_settings.comment_prefix,
                     ):
                         # Capture the first line (YAML ID) as content
-                        if entry.show_id or entry.summary_id or entry.step_id:
+                        if entry.process_id():
+                            # Line ending with a colon
                             if line[-1] == ":":
                                 line = line[:-1]
+                            # In-line sls in a top file
+                            elif entry.topfile_id and ":" in line:
+                                fields = line.split(":")
+                                entry.add_include(fields[1].strip())
+                                line = fields[0].strip()
 
-                            if entry.summary_id or entry.step_id:
+                            if entry.prepend_id:
                                 # Prepend with a newline so the summary is correctly identified later
                                 entry.prepend_line("")
+
+                                # Remove any leading whitespace as the summary has to be left-justified
+                                if self.app.config.autosaltsls_indented_comments:
+                                    line = line.lstrip(" ")
+
                                 entry.prepend_line(line)
+
+                                # Clear the process_id flag and force another line to be read so we can parse the
+                                # block entries as includes
+                                if entry.topfile_id:
+                                    entry.process_id(False)
+                                    continue
                             else:
                                 entry.append_line(line)
 
-                        # Read all the include entries (flag set by directive above)
-                        elif entry.include:
+                        # Read all the include or topfile entries (flag set by directive above)
+                        elif entry.include or entry.topfile_id:
                             if "include:" in line:
                                 included = True
                                 continue
-                            elif included and line and not line.isspace():
+                            elif (
+                                (included or entry.topfile_id)
+                                and line
+                                and not line.isspace()
+                            ):
                                 # Use regex to match an include entry and store it
                                 # First non-match will trigger block end
-                                match = re.search("^\s+-\s+([\w\-.]+)", line)
+                                match = re.search("^\s+-\s+([\s\w\-.:]+)", line)
                                 if match:
                                     text = match.group(1)
 
+                                    if "match:" in text:
+                                        entry.match_type = text.replace(
+                                            "match:", ""
+                                        ).strip()
+                                        continue
                                     if text.startswith("."):
                                         text = "{0} <{1}{2}>".format(
                                             text,
@@ -306,13 +367,23 @@ class AutoSaltSLS(object):
                                     entry.add_include(text)
                                     continue
 
+                                # Skip any jinja directives within the includes
+                                if "{%" in line or "{%" in line:
+                                    continue
+
                         # Add the entry to the main list
                         self.add_entry(entry)
                         entry = None
+
                         continue
 
                     # Any other comment line within an active block is part of the entry
-                    if entry and line.startswith(self.source_settings.comment_prefix):
+                    if entry and self._check_line_startswith(
+                        line, self.source_settings.comment_prefix
+                    ):
+                        if self.app.config.autosaltsls_indented_comments:
+                            line = line.lstrip(" ")
+
                         line = line.replace(self.source_settings.comment_prefix, "")
 
                         if self.app.config.autosaltsls_remove_first_space:
@@ -454,6 +525,21 @@ class AutoSaltSLS(object):
     #
     # Private functions
     #
+    def _check_line_startswith(
+        self, line, pattern,
+    ):
+        """
+        Check if a line starts with a pattern, optionally ignoring leading spaces if ``autosaltsls_indented_comments``
+        is set.
+        """
+        if self.app.config.autosaltsls_indented_comments:
+            line = line.lstrip(" ")
+
+        if line.startswith(pattern):
+            return True
+
+        return False
+
     @staticmethod
     def _parse_name(basename):
         # Replace the path separator with a dot
@@ -478,6 +564,15 @@ class AutoSaltSLSEntry(object):
         Directives are run-time changes to apply to an entry. They are specified on the document prefix line as a
         comma-separated list
 
+        environment
+            Use the following line as a topfile environment (e.g. 'base', 'production')
+
+        hidden
+            Do not generate documentation for this file, also sets ``ignore``
+
+        ignore
+            Immediately stop processing the file. Any entries already found are returned as normal
+
         include
             The lines following this comment block will be an ``include:`` section and should be parsed to add the YAML
             list data items to the ``includes`` list
@@ -494,22 +589,30 @@ class AutoSaltSLSEntry(object):
 
         summary_id
              Read the first line following this comment block and add it as the entry summary
+
+         topfile_id
+            Read the first line following this comment block and add it as the entry summary then process the following
+             lines to extract the matching criteria and sub-lines for generating cross-references
     """
 
     def __init__(self, text=None):
         self.lines = []
         self.includes = []
+        self.match_type = None
 
         # Directives
+        self.environment = False
         self.include = False
         self.show_id = False
         self.step = False
         self.step_id = False
         self.summary_id = False
+        self.topfile_id = False
 
         # Internal properties
         self._summary = None
         self._content = None
+        self._process_id = None
 
         # Initialise with any text we've been given
         if text is not None:
@@ -520,12 +623,13 @@ class AutoSaltSLSEntry(object):
 
     def add_include(self, include):
         """
-        Add an include statement to the list.
+        Add an include statement to the list and set the ``include`` property.
 
         include
             Include statement to add
         """
-        self.includes.append(include)
+        self.includes.append(include.strip(" "))
+        self.include = True
 
     @property
     def annotated_text(self):
@@ -588,6 +692,18 @@ class AutoSaltSLSEntry(object):
         """
         return self.step or self.step_id
 
+    @property
+    def prepend_id(self):
+        """
+        Return whether this entry should prepend the YAML id to the output
+
+        :return: bool
+        """
+        if self.summary_id or self.step_id or self.environment or self.topfile_id:
+            return True
+
+        return False
+
     def prepend_line(self, text):
         """
         Add some text to the start of the content line list.
@@ -600,6 +716,27 @@ class AutoSaltSLSEntry(object):
         # Reset the internal content property
         self._content = None
 
+    def process_id(self, new_val=None):
+        """
+        Flag to show whether this entry requires YAML id processing. If not set and no ``new_val`` supplied, then
+        the flag value is calculated based on other directive attributes.
+
+        :return: bool
+        """
+        if new_val is not None:
+            self._process_id = new_val
+        elif self._process_id is None:
+            if (
+                self.show_id
+                or self.summary_id
+                or self.step_id
+                or self.environment
+                or self.topfile_id
+            ):
+                self._process_id = True
+
+        return self._process_id
+
     @property
     def summary(self):
         """
